@@ -1,33 +1,12 @@
+from .provider.provider import Provider
+from .provider.vault import Vault
 import os
 import glob
-import sys
+import re
 import json
-import requests
-import hvac  # type: ignore
 import collections.abc as collections
-from typing import Dict, List, Tuple, Type, Union, Optional, MutableMapping, Text, Any, NamedTuple
+from typing import Dict, List, Type, Union, Optional, MutableMapping, Text, Any
 import yaml
-if sys.version_info >= (3, 8):
-    from typing import TypedDict
-else:
-    from typing_extensions import TypedDict
-
-
-class CACertClient(NamedTuple):
-    client_cert_path: str
-    client_key_path: str
-
-
-class HVAC_ClientConfig(TypedDict):
-    url: Optional[str]
-    token: str
-    cert: Union[None, CACertClient]
-    verify: Union[None, bool]
-
-
-class HVAC_ClientAuthentication(TypedDict):
-    role: str
-    jwt: str
 
 
 class Gestalt:
@@ -54,7 +33,10 @@ class Gestalt:
                                            float]] = dict()
         self.__conf_defaults: Dict[Text, Union[List[Any], Text, int, bool,
                                                float]] = dict()
-        self.__vault_paths: List[Tuple[Union[str, None], str]] = []
+        self.__providers: Dict[str, Provider] = dict()
+        self.__secret_map: Dict[str, List[str]] = {}
+        self.regex_pattern = re.compile(
+            r"^ref\+([^\+]*)://([^(\+)]+)\#([^\+]+)?$")
 
     def __flatten(
         self,
@@ -172,6 +154,61 @@ class Gestalt:
         self.__conf_data = self.__flatten(self.__conf_data,
                                           sep=self.__delim_char)
 
+        self.__parse_keys()
+        self.__interpolate_keys()
+
+    def __parse_keys(self) -> None:
+        """Parses the keys in the configuration data.
+
+        Raises:
+            RuntimeError: If the configuration data is not valid
+        """
+        for k, v in self.__conf_data.items():
+            if not isinstance(v, str):
+                continue
+            m = self.regex_pattern.search(v)
+            if m is None:
+                continue
+            if m.group(1) not in self.__providers:
+                raise RuntimeError(
+                    "Provider not configured yet expect to be used")
+            if v in self.__secret_map:
+                self.__secret_map[v].append(k)
+            else:
+                self.__secret_map.update({v: [k]})
+
+    def configure_provider(self, provider_name: str,
+                           provider: Provider) -> None:
+        """Configures a provider for use in the library.
+
+        Args:
+            provider_name (str): The name of the provider to configure
+            provider (Provider): The provider to configure
+
+        Raises:
+            TypeError: If the provider is not an instance of the Provider class
+        """
+        if provider_name == "vault" and isinstance(provider, Vault):
+            self.__providers.update({"vault": provider})
+        else:
+            raise TypeError("Provider provider is not supported")
+
+    def __interpolate_keys(self) -> None:
+        """Interpolates the keys in the configuration data.
+        """
+        for path, v in self.__secret_map.items():
+            m = self.regex_pattern.search(path)
+            if m is not None:
+                provider = self.__providers[m.group(1)]
+                for config_key in v:
+                    secret = provider.get(key=config_key,
+                                          path=m.group(2),
+                                          filter=m.group(3))
+                    self.__conf_data.update({config_key: secret})
+
+        self.__conf_data = self.__flatten(self.__conf_data,
+                                          sep=self.__delim_char)
+
     def auto_env(self) -> None:
         """Auto env provides sane defaults for using environment variables
 
@@ -543,79 +580,3 @@ class Gestalt:
         ret.update(self.__conf_data)
         ret.update(self.__conf_sets)
         return str(json.dumps(ret, indent=4))
-
-    def add_vault_config_provider(
-            self, client_config: HVAC_ClientConfig,
-            auth_config: Optional[HVAC_ClientAuthentication]) -> None:
-        """Initialized vault client and authenticates vault
-
-        Args:
-            client_config (HVAC_ClientConfig): initializes vault. URL can be set in VAULT_ADDR
-                environment variable, token can be set to VAULT_TOKEN environment variable.
-                These will be picked by default if not set to empty string
-            auth_config (HVAC_ClientAuthentication): authenticates the initialized vault client
-                with role and jwt string from kubernetes
-        """
-        if bool(client_config) == False:
-            raise TypeError("Gestalt Error: client config is empty for Vault")
-        client_config['url'] = os.environ.get("VAULT_ADDR")
-        print(client_config)
-        if not client_config['url']:
-            raise RuntimeError('Gestalt Error: VAULT_ADDR is missing')
-        if client_config['token'] == "":
-            client_config['token'] == os.environ.get("VAULT_TOKEN", "")
-        if client_config['verify']:
-            verify = True
-        else:
-            verify = False
-
-        self.vault_client: hvac.Client = hvac.Client(client_config['url'],
-                                                     client_config['token'],
-                                                     client_config['cert'],
-                                                     verify=verify)
-
-        if auth_config:
-            try:
-                self.vault_client.auth_kubernetes(\
-                    role=auth_config['role'],
-                    jwt=auth_config['jwt']
-                )
-            except requests.exceptions.ConnectionError as err:
-                print(
-                    "Gestalt Error: Couldn't connect to Vault. Maybe missing VAULT_ADDR?"
-                )
-
-    def add_vault_secret_path(self,
-                              path: str,
-                              mount_path: Optional[str] = None) -> None:
-        """Adds a vault secret with key and path to gestalt
-
-        Args:
-            path (str): The path to the secret in vault cluster
-            mount_path ([type], optional): The mount_path for a non-default secret
-                mount. Defaults to Optional[str].
-        """
-        mount_path = mount_path if mount_path != None else "secret"
-
-        self.__vault_paths.append((mount_path, path))
-
-    def fetch_vault_secrets(self) -> None:
-        """Fetches client secrets from vault first checks the path provided 
-        """
-        if len(self.__vault_paths) <= 0:
-            return
-        print("Fetching secrets from VAULT")
-        for vault_path in self.__vault_paths:
-            mount_path = str(vault_path[0])
-            secret_path = vault_path[1]
-            try:
-                secret_token = self.vault_client.secrets.kv.v2.read_secret_version(
-                    mount_point=str(mount_path), path=secret_path)
-                self.__conf_data.update(secret_token['data']['data'])
-            except hvac.exceptions.InvalidPath as err:
-                raise RuntimeError(
-                    "Gestalt Error: The secret path or mount is set incorrectly"
-                )
-            except requests.exceptions.ConnectionError as err:
-                raise RuntimeError(
-                    "Gestalt Error: Gestalt couldn't connect to Vault")
